
# imports.
from classes.config import *
from apps.packages.packages import inc_package_manager

# the user object class.
class User(Object):
	def __init__(self):

		# defaults.
		Object.__init__(self)

		# sys attributes.
		self.__username__ = None
		self.__email__ = None
		self.__name__ = None
		self.__api_key__ = None
		self.__password__ = None
		self.__authenticated__ = False
		self.__activated__ = False

		#

	# signup to api.vandenberghinc.com
	def signup(self,
		# the user's username.
		username=None,
		# the user's email.
		email=None,
		# the user's name.
		name=None,
		# the users's password.
		password=None,
		# the users's verify password for signup.
		verify_password=None,
		# the verification code send to the email.
		code=None,
	):

		# check signed in.
		if self.authenticated:
			return dev0s.response.error("The current user is already authenticated.")
			
		# request.
		response = self.__request__("/requests/accounts/signup/", {
			"username":username,
			"email":email,
			"name":name,
			"password":password,
			"verify_password":verify_password,
			"code":code,
		})
		if response.success:
			self.__username__ = username
			self.__email__ = email
			self.__name__ = name
			self.__password__ = password
		return response

		#

	# signin to api.vandenberghinc.com
	def signin(self,
		# the user's username.
		username=None,
		# the users's password.
		password=None,
	):

		# check signed in.
		if self.authenticated:
			return dev0s.response.success("The current user is already authenticated.")
			
		# request.
		response = self.__request__("/requests/accounts/signin/", {
			"username":username,
			"password":password,
		})
		if response.success:
			self.__authenticated__ = True
			self.__activated__ = response.activated
			self.__username__ = username
			self.__password__ = password
			self.__api_key__ = response.api_key
			self.__name__ = response.name
			self.__email__ = response.email
			inc_package_manager.api_key = response.api_key
		return response

		#

	# signout.
	def signout(self,
		# the user's username.
		username=None,
		# the users's password.
		password=None,
	):

		# check signed in.
		if not self.authenticated:
			return dev0s.response.success("The current user is not authenticated.")
			
		# request.
		username = self.username
		self.__username__ = None
		self.__email__ = None
		self.__name__ = None
		self.__api_key__ = None
		self.__password__ = None
		self.__authenticated__ = False
		self.__activated__ = False
		inc_package_manager.api_key = None
		return dev0s.response.success(f"Successfully signed out {username}.")

		#

	# edit the user's settings.
	def edit(self,
		# the new email.
		email=None,
		# the new name.
		name=None,
	):

		# check signed in.
		if not self.authenticated:
			return dev0s.response.error("The current user is not authenticated.")

		# check params.
		if [email,name] == [None,None]:
			return dev0s.response.error("Define one of the following parameters: [email, name].")

		# request.
		response = self.__request__("/requests/accounts/edit/", {
			"email":email,
			"name":name,
		})
		if response.success:
			if email != None: self.__email__ = email
			if name != None: self.__name__ = name
		return response

		#

	# edit the user's password.
	def edit_password(self,
		# the current password (str).
		password=None,
		# the new password (str).
		new_password=None,
		# the verify password (str).
		verify_password=None,
	):

		# check signed in.
		if not self.authenticated:
			return dev0s.response.error("The current user is not authenticated.")

		# check params.
		response = dev0s.response.parameters.check(
			parameters={
				"password:str":password,
				"new_password:str":new_password,
				"verify_password:str":verify_password,
			})
		if not response.success: return response

		# check current.
		if password != self.__password__:
			return dev0s.response.error("Provided an incorrect password.")

		# request.
		response = self.__request__("/requests/accounts/edit_password/", {
			"password":password,
			"new_password":new_password,
			"verify_password":verify_password,
		})
		if response.success:
			self.__password__ = password
		return response

		#

	# send verfication code.
	def send_code(self,
		# the user's username (str).
		username=None,
		# the verification code's mode.
		mode="reset_password",
	):

		# check params.
		response = dev0s.response.parameters.check(
			parameters={
				"username:str":username,
				"mode:str":mode,
			})
		if not response.success: return response

		# request.
		return self.__request__("/requests/accounts/send_code/", {
			"username":username,
			"mode":mode,
		})

		#

	# reset the user's password.
	def reset_password(self,
		# the user's username (str).
		username=None,
		# the verification code send to the email (str).
		code=None,
		# the new password (str).
		password=None,
		# the verify password (str).
		verify_password=None,
	):

		# check signed in.
		if self.authenticated:
			return dev0s.response.error("The current user is already authenticated.")

		# check params.
		response = dev0s.response.parameters.check(
			parameters={
				"username:str":username,
				"code:str":code,
				"password:str":password,
				"verify_password:str":verify_password,
			})
		if not response.success: return response

		# request.
		response = self.__request__("/requests/accounts/reset_password/", {
			"username":username,
			"code":code,
			"password":password,
			"verify_password":verify_password,
		})
		if response.success:
			self.__password__ = password
		return response

		#

	# activate a newly created account.
	def activate(self,
		# the verification code send to the email (str).
		code=None,
	):

		# check signed in.
		if not self.authenticated:
			return dev0s.response.error("The current user is not authenticated.")

		# check already activated.
		if self.activated:
			return dev0s.response.error("The current user is already activated.")

		# check params.
		response = dev0s.response.parameters.check(
			parameters={
				"code:str":code,
			})
		if not response.success: return response

		# request.
		response = self.__request__("/requests/accounts/activate/", {
			"username":self.username,
			"code":code,
		})
		if response.success:
			self.__activated__ = True
		return response

		#

	# properties.
	@property
	def username(self):
		return self.__username__
	@property
	def email(self):
		return self.__email__
	@property
	def name(self):
		return self.__name__
	@property
	def api_key(self):
		return self.__api_key__
	@property
	def authenticated(self):
		return self.__authenticated__
	@property
	def activated(self):
		return self.__activated__
	@property
	def info(self):
		# public info.
		return {
			"username":self.username,
			"email":self.email,
			"name":self.name,
			"authenticated":self.authenticated,
			"activated":self.activated,
		}
	
	# system functions.
	def __request__(self, url="/", data={}, json=True, log_level=dev0s.defaults.options.log_level):
		url = f"api.vandenberghinc.com/{gfp.clean(url, remove_first_slash=True, remove_last_slash=True)}/"
		data["api_key"] = self.api_key
		return dev0s.requests.get(url=url, data=data, serialize=json)

# the initialized user object.
user = User()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# inc imports.
from dev0s.shortcuts import *

# alias.
ALIAS = "inc-package-manager"

# source.
SOURCE = Directory(dev0s.defaults.source_path(__file__, back=3))
OS = dev0s.defaults.operating_system(supported=["linux", "macos"])

# checks.
dev0s.defaults.alias(alias=ALIAS, executable=f"{SOURCE}")

# production settings.
# do not deploy with production disabled.
PRODUCTION = dev0s.env.get("PRODUCTION", format=bool, default=True)

# database.
DATABASE = Database(f"/etc/{ALIAS}/")
if not Files.exists(str(DATABASE)[:-1]):
	print(f"{color.orange}Root permissions{color.end} required to create database {DATABASE}.")
	os.system(f"sudo mkdir {DATABASE} && sudo chown -R {dev0s.defaults.vars.user}:{dev0s.defaults.vars.group} {DATABASE} && sudo chmod -R 770 {DATABASE}")

# check dirs.
for dir in [
	Files.join(SOURCE, "__defaults__"),
	Files.join(SOURCE, "__defaults__/env"),
]:
	if not Files.exists(dir):  Files.create(dir, directory=True)

# limit website init.
if dev0s.system.env.get("INC_PACKAGE_MANAGER_API", format=bool, default=False):

	# import w3bsite.
	import w3bsite 
	
	# website.
	website = w3bsite.Website(
		# main.
		root=str(SOURCE),
		database=str(DATABASE),
		library=f"/usr/local/lib/{ALIAS}",
		domain="127.0.0.1:52940",
		name="IncPackageManager",
		remote=None,
		# the template data (only required when running the website) (overwrites the w3bsite template data keys).
		template_data={
			"COLORS":{
				"white":"#FAFAFA",
				"light_white":"#E9F0FD",
				"grey":"#E5E5E5",
				"light_grey":"#D6D6D6",
				"dark_grey":"#424242",
				"blue":"#5A8FE6",
				"purple":"#323B83",#"#B32FCA",
				#"purple":"#9B00AA",
				"red":"#FD304E",
				"pink":"#F62B7D",
				"orange":"#FF8800",
				"green":"#006633",
				"darkest":"#1F2227",
				"darker": "#20242A",
				"dark": "#262B30",
				# background color.
				"topbar":"#323B83",#1F2227", #"#FAFAFA",
				"background":"#323B83",#"#E7E9EF", #"#FAFAFA",
				"topbar_darkmode":"#1F2227",#1F2227", #"#FAFAFA",
				"background_darkmode":"#1F2227",#"#E7E9EF", #"#FAFAFA",
				# elements.
				"widgets":"#FAFAFA",
				"widgets_reversed":"#323B83",#"#1F2227",
				"widgets_darkmode":"#323B83",
				"widgets_reversed_darkmode":"#20242A",#"#1F2227",
				# text.
				"text":"#1F2227",
				"text_reversed":"#FAFAFA",
				"text_darkmode":"#FAFAFA",
				"text_reversed_darkmode":"#FAFAFA",
				# input & textareas.
				"input_txt":"#1F2227",
				"input_txt_reversed":"#FAFAFA",
				"input_bg":"#FAFAFA",
				"input_bg_reversed":"#323B83",
				# buttons.
				"button_txt":"#FAFAFA",
				"button_txt_reversed":"#1F2227",
				"button_bg":"#323B83",
				"button_bg_reversed":"#FAFAFA",
				# custom colors.
				# ...
			},
		},
		# custom styling.
		styling={
			"LEFTBAR_WIDTH":"280px", # px
			"RIGHTBAR_WIDTH":"280px", # px
			"TOPBAR_HEIGHT":"50px", #px
		},
		# 	the organization name.
		organization="VanDenBerghInc",
		# aes.
		#aes_passphrase=CONFIG["aes"]["master"],
		# options.
		namecheap_enabled=False,
		firebase_enabled=False,
		stripe_enabled=False,
		email_enabled=False,
		interactive=False,
		_2fa=False,
		production=PRODUCTION,
	)

#




# imports.
from classes.config import *
from apps.accounts.user import user
	
# sign in.
class SignIn(w3bsite.views.View):
	def __init__(self):
		w3bsite.views.View.__init__(self, f"accounts/", "signin", website=website)
	def view(self, request):
		return self.render(request, {
			"USER":user.info,
		})
class SignInCopy(w3bsite.views.View):
	def __init__(self):
		w3bsite.views.View.__init__(self, f"accounts/", "login", website=website, html="accounts/html/signin.html")
	def view(self, request):
		return self.render(request, {
			"USER":user.info,
		})

# sign up.
class SignUp(w3bsite.views.View):
	def __init__(self):
		w3bsite.views.View.__init__(self, f"accounts/", "signup", website=website)
	def view(self, request):
		return self.render(request, {
			"USER":user.info,
		})

# reset password.
class Reset(w3bsite.views.View):
	def __init__(self):
		w3bsite.views.View.__init__(self, f"accounts/", "reset", website=website)
	def view(self, request):
		return self.render(request, {
			"USER":user.info,
		})

# activate account.
class Activate(w3bsite.views.View):
	def __init__(self):
		w3bsite.views.View.__init__(self, f"accounts/", "activate", website=website)
	def view(self, request):
		if not user.authenticated:
			return self.render(request, html="accounts/html/signin.html", template_data={
				"USER": user.info
			})
		else:
			return self.render(request, {
				"USER":user.info,
			})

# the updating view.
class Account(w3bsite.views.View):
	def __init__(self):
		w3bsite.views.View.__init__(self, "accounts/", "account", website=website)
	def view(self, request):
		if not user.authenticated:
			return self.render(request, html="accounts/html/signin.html", template_data={
				"USER": user.info
			})
		elif not user.activated:
			return self.render(request, html="accounts/html/activate.html", template_data={
				"USER": user.info
			})
		else:
			return self.render(request, {
				"USER": user.info
			})

		#

# url patterns.
urlpatterns = w3bsite.views.build_urls([
	SignIn(),
	SignInCopy(),
	SignUp(),
	Reset(),
	Activate(),
	Account(),
])



# classes imports.
from classes.config import *
from apps.packages.packages import inc_package_manager

# install package.
class Install(w3bsite.views.Request):
	def __init__(self):
		w3bsite.views.Request.__init__(self, 
			"requests/packages/", "install", 
			website=website,
			auth_required=False, 
			root_required=False, )
	def request(self, request):
		
		# check authenicated.
		parameters, response = self.parameters.get(request, [
			"package",])
		if not response.success: return response

		# requests.
		return inc_package_manager.install(parameters.package)
		
		#

# uninstall package.
class Uninstall(w3bsite.views.Request):
	def __init__(self):
		w3bsite.views.Request.__init__(self, 
			"requests/packages/", "uninstall", 
			website=website,
			auth_required=False, 
			root_required=False, )
	def request(self, request):
		
		# check authenicated.
		parameters, response = self.parameters.get(request, [
			"package",])
		if not response.success: return response

		# requests.
		return inc_package_manager.uninstall(parameters.package)
		
		#

# retrieve the package version.
class Version(w3bsite.views.Request):
	def __init__(self):
		w3bsite.views.Request.__init__(self, 
			"requests/packages/", "version", 
			website=website,
			auth_required=False, 
			root_required=False, )
	def request(self, request):
		
		# check authenicated.
		parameters, response = self.parameters.get(request, [
			"package"])
		if not response.success: return response
		optional_parameters, _ = self.parameters.get(request, {
			"remote":True,
		})

		# requests.
		return inc_package_manager.version(parameters.package, remote=optional_parameters.remote)
		
		#

# update package.
class Update(w3bsite.views.Request):
	def __init__(self):
		w3bsite.views.Request.__init__(self, 
			"requests/packages/", "update", 
			website=website,
			auth_required=False, 
			root_required=False, )
	def request(self, request):
		
		# check authenicated.
		parameters, response = self.parameters.get(request, [
			"package",])
		if not response.success: return response

		# requests.
		return inc_package_manager.update(parameters.package)
		
		#


# url patterns.
urlpatterns = w3bsite.views.build_urls([
	Install(),
	Uninstall(),
	Version(),
	Update()
])

#